open Printf
open Types
open Util

    
let process (line : string) =
  let linebuf = Lexing.from_string line in
  try
    (* Run the parser on this line of input *)
    let vals = Parser.main Lexer.token linebuf in
    let model =  values_to_model vals in
     print_model model ;   
    if model.count_w > 0 then Some model  else None
  (*  print_model_gv model *)
  with
  | Lexer.Error msg ->
(*    fprintf stderr "### LEXER ERROR ###: %s%!" msg ; *)
    None
  | Parser.Error ->
(*       fprintf stderr "### PARSER ERROR ###: At offset %d: syntax error.\n%!" (Lexing.lexeme_start linebuf) ; *)
     None
	
let process_opt (optional_line : string option) =
  match optional_line with
  | None ->
      None
  | Some line ->
       process line 
	
let rec repeat lastModel_opt  channel =
  (* Attempt to read one line. *)
  (* lastModel_opt stores the last valid model, if defined *)
  let (optional_line, continue) = Lexer.line channel in
  let model_opt = process_opt optional_line in   
  let update_lastModel_opt =
    match model_opt with
    | Some model -> model_opt
    | None ->       lastModel_opt in  
  if continue then
    repeat update_lastModel_opt  channel
  else     update_lastModel_opt 
      
let () =
  let model_opt = repeat None (Lexing.from_channel  stdin) in
  match model_opt with
  | Some model  -> print_model_gv model
  | None ->  printf   "### Not a model###\n"  
    
    
    
    
